@@ -1,4 +1,4 @@
-include Caml
+include Stdlib
 
 (* At Jane Street, the OCaml stdlib is patched to define [Pervasives.raise] as the
    ["%reraise"] primitive. We do this as the compiler is currently not good enough at
@@ -12,6 +12,7 @@ include Caml
    [[1] http://caml.inria.fr/mantis/view.php?id=6556
 *)
 external raise : exn -> 'a = "%reraise"
+external ignore : ('a[@local_opt]) -> unit = "%ignore"
 
 [%%if ocaml_version < (4, 12, 0)]
 
@@ -19,9 +20,19 @@ let __FUNCTION__ = "<__FUNCTION__ not supported before OCaml 4.12>"
 
 [%%endif]
 
-[%%if ocaml_version >= (5, 0, 0)]
+external __LOC_OF__ : ('a[@local_opt]) -> (string * 'a[@local_opt]) = "%loc_LOC"
+external __LINE_OF__ : ('a[@local_opt]) -> (int * 'a[@local_opt]) = "%loc_LINE"
 
-external ( & ) : bool -> bool -> bool = "%sequand"
-external ( or ) : bool -> bool -> bool = "%sequor"
+external __POS_OF__
+  :  ('a[@local_opt])
+  -> ((string * int * int * int) * 'a[@local_opt])
+  = "%loc_POS"
 
-[%%endif]
+external ( |> ) : 'a -> (('a -> 'b)[@local_opt]) -> 'b = "%revapply"
+external ( @@ ) : (('a -> 'b)[@local_opt]) -> 'a -> 'b = "%apply"
+external int_of_char : (char[@local_opt]) -> int = "%identity"
+
+external format_of_string
+  :  (('a, 'b, 'c, 'd, 'e, 'f) format6[@local_opt])
+  -> (('a, 'b, 'c, 'd, 'e, 'f) format6[@local_opt])
+  = "%identity"
